@@ -140,12 +140,17 @@ data_entry_refresh_display (DataEntry *wid)
 }
 
 void
-data_entry_update_with_value (DataEntry *de, const GdaValue *value, gboolean value_as_init)
+data_entry_update_with_value (DataEntry *de, 
+			      const GdaValue *value, GdaValueType fallback_orig_type,
+			      gboolean value_as_init)
 {
 	g_return_if_fail (de && IS_DATA_ENTRY (de));
 	g_return_if_fail (de->priv->data_handler);
 	
 	data_handler_widget_update (de->priv->data_handler, GTK_WIDGET (de), value, value_as_init);
+	if ((de->priv->orig_type == GDA_VALUE_TYPE_NULL) ||
+	    (de->priv->orig_type == GDA_VALUE_TYPE_UNKNOWN))
+		de->priv->orig_type = fallback_orig_type;
 }
 
 void
@@ -356,7 +361,8 @@ static GtkWidget *
 build_actions_menu (DataEntry *de)
 {
 	GtkWidget *menu, *mitem;
-	gchar *str, *name = NULL;
+	gchar *str;
+	const gchar *name = NULL;
 	gboolean nullval = FALSE;
 	gboolean defval = de->priv->is_default_possible;
 	gboolean reset = FALSE;
@@ -365,17 +371,34 @@ build_actions_menu (DataEntry *de)
 
 	/* which menu items do make sensitive ? */
 	if (!de->priv->value_required) {
-		if ((de->priv->query_field && !de->priv->value_is_null) ||
-		    (de->priv->value_is_null && de->priv->value_is_defaultval)) {
-			name = de->priv->query_field->name;
-					
-			if (de->priv->query_field->field_type == QUERY_FIELD_FIELD) {
+		gboolean tmp = TRUE;
+
+		if (de->priv->value_is_null && de->priv->value_is_defaultval)
+			tmp = FALSE;
+
+		if (de->priv->query_field && !de->priv->value_is_null) {
+			name = query_field_get_name (de->priv->query_field);
+
+			if (query_field_get_ftype (de->priv->query_field) == QUERY_FIELD_FIELD) {
 				DbField *field;
 				field = query_field_field_get_db_field (de->priv->query_field);
 				if (field) 
 					nullval = field->null_allowed;
 			}
 		}
+
+		nullval = tmp;
+		/* if ((de->priv->query_field && !de->priv->value_is_null) || */
+/* 		    (de->priv->value_is_null && de->priv->value_is_defaultval)) { */
+/* 			name = de->priv->query_field->name; */
+					
+/* 			if (de->priv->query_field->field_type == QUERY_FIELD_FIELD) { */
+/* 				DbField *field; */
+/* 				field = query_field_field_get_db_field (de->priv->query_field); */
+/* 				if (field)  */
+/* 					nullval = field->null_allowed; */
+/* 			} */
+/* 		} */
 	}
 
 	if (defval && de->priv->value_is_defaultval)
