@@ -1,6 +1,6 @@
 /* query.c
  *
- * Copyright (C) 1999 - 2002 Vivien Malerba
+ * Copyright (C) 1999 - 2003 Vivien Malerba
  * Copyright (C) 2001 - 2002 Fernando Martins
  *
  * This program is free software; you can redistribute it and/or
@@ -57,13 +57,11 @@ enum
 	FIELD_PRINT_MODIFIED,
 	FIELD_NAME_MODIFIED,
 	FIELD_ALIAS_MODIFIED,
+	FIELDS_ORDER_CHANGED,
 	JOIN_CREATED,
 	JOIN_DROPPED,
 	JOIN_MODIFIED,
-	COND_CREATED,
-	COND_DROPPED,
-	COND_MODIFIED,
-	COND_MOVED,
+	COND_CHANGED,
 	ENV_CREATED,
 	ENV_DROPPED,
 	QUERY_VIEW_ADDED,
@@ -76,7 +74,7 @@ enum
 };
 
 static gint query_signals[LAST_SIGNAL] =
-	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
 
 guint
@@ -181,6 +179,14 @@ query_class_init (QueryClass * class)
 			      NULL, NULL,
 			      marshal_VOID__POINTER, G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
+	query_signals[FIELDS_ORDER_CHANGED] =
+		g_signal_new ("fields_order_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (QueryClass, fields_order_changed),
+			      NULL, NULL,
+			      marshal_VOID__VOID, G_TYPE_NONE,
+			      0);
 	query_signals[JOIN_CREATED] =
 		g_signal_new ("join_created",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -205,38 +211,15 @@ query_class_init (QueryClass * class)
 			      NULL, NULL,
 			      marshal_VOID__POINTER, G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
-	query_signals[COND_CREATED] =
-		g_signal_new ("cond_created",
+
+	query_signals[COND_CHANGED] =
+		g_signal_new ("cond_changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (QueryClass, cond_created),
+			      G_STRUCT_OFFSET (QueryClass, cond_changed),
 			      NULL, NULL,
-			      marshal_VOID__POINTER, G_TYPE_NONE, 1,
-			      G_TYPE_POINTER);
-	query_signals[COND_DROPPED] =
-		g_signal_new ("cond_dropped",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (QueryClass, cond_dropped),
-			      NULL, NULL,
-			      marshal_VOID__POINTER, G_TYPE_NONE, 1,
-			      G_TYPE_POINTER);
-	query_signals[COND_MODIFIED] =
-		g_signal_new ("cond_modified",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (QueryClass, cond_modified),
-			      NULL, NULL,
-			      marshal_VOID__POINTER, G_TYPE_NONE, 1,
-			      G_TYPE_POINTER);
-	query_signals[COND_MOVED] =
-		g_signal_new ("cond_moved",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (QueryClass, cond_moved),
-			      NULL, NULL,
-			      marshal_VOID__POINTER, G_TYPE_NONE, 1,
-			      G_TYPE_POINTER);
+			      marshal_VOID__VOID, G_TYPE_NONE,
+			      0);
 	
 	query_signals[ENV_CREATED] =
 		g_signal_new ("env_created",
@@ -310,36 +293,21 @@ query_class_init (QueryClass * class)
 			      G_TYPE_POINTER);
 
 
-	class->field_created =
-		(void (*)(Query * q, QueryField * field)) m_changed;
-	class->field_dropped =
-		(void (*)(Query * q, QueryField * field)) m_changed;
-	class->field_modified =
-		(void (*)(Query * q, QueryField * field)) m_changed;
-	class->field_print_modified =
-		(void (*)(Query * q, QueryField * field)) m_changed;
-	class->field_name_modified =
-		(void (*)(Query * q, QueryField * field)) m_changed;
-	class->field_alias_modified =
-		(void (*)(Query * q, QueryField * field)) m_changed;
+	class->field_created = (void (*)(Query * q, QueryField * field)) m_changed;
+	class->field_dropped = (void (*)(Query * q, QueryField * field)) m_changed;
+	class->field_modified =	(void (*)(Query * q, QueryField * field)) m_changed;
+	class->field_print_modified = (void (*)(Query * q, QueryField * field)) m_changed;
+	class->field_name_modified = (void (*)(Query * q, QueryField * field)) m_changed;
+	class->field_alias_modified = (void (*)(Query * q, QueryField * field)) m_changed;
 
-	class->join_created = 
-		(void (*)(Query * q, QueryJoin * qj))	m_changed;
-	class->join_dropped = 
-		(void (*)(Query * q, QueryJoin * qj)) m_changed;
-	class->join_modified = 
-		(void (*)(Query * q, QueryJoin * qj)) m_changed;
+	class->join_created = (void (*)(Query * q, QueryJoin * qj)) m_changed;
+	class->join_dropped = (void (*)(Query * q, QueryJoin * qj)) m_changed;
+	class->join_modified = (void (*)(Query * q, QueryJoin * qj)) m_changed;
 	class->changed = NULL;
+	class->fields_order_changed = (void (*)(Query * q)) m_changed;;
 	class->name_changed = NULL;
 	class->type_changed = (void (*)(Query * q)) m_changed;
-	class->cond_created =
-		(void (*)(Query * q, QueryCond * node)) m_changed;
-	class->cond_dropped =
-		(void (*)(Query * q, QueryCond * node)) m_changed;
-	class->cond_modified =
-		(void (*)(Query * q, QueryCond * node)) m_changed;
-	class->cond_moved =
-		(void (*)(Query * q, QueryCond * node)) m_changed;
+	class->cond_changed = (void (*)(Query * q)) m_changed;
 
 	class->query_view_added = NULL;
 	class->query_view_removed = NULL;
@@ -617,10 +585,6 @@ query_activate_all_fields (Query *q)
 	list = q->fields;
 	while (list) {
 		query_field_activate (QUERY_FIELD (list->data));
-		if (! query_field_get_activated (QUERY_FIELD (list->data)))
-			g_warning ("QueryField %s.%s can't be activated (%s line %d)\n", 
-				   q->name, query_field_get_name (QUERY_FIELD (list->data)),
-				   __FILE__, __LINE__);
 
 		list = g_slist_next (list);
 	}
@@ -642,9 +606,6 @@ query_fields_activated   (Query *q)
 	list = q->fields;
 	while (list && retval) {
 		retval = query_field_get_activated (QUERY_FIELD (list->data));
-		/*if (! retval)
-			g_warning ("QueryField %s.%s is not activated (%s line %d)\n", 
-				   q->name, QUERY_FIELD (list->data)->name, __FILE__, __LINE__);*/
 
 		list = g_slist_next (list);
 	}
@@ -1691,7 +1652,8 @@ query_del_field (Query *q, QueryField *field)
 	/* disconnect the "alias_modified" signal */
 	g_signal_handlers_disconnect_by_func (G_OBJECT (field),
 					      G_CALLBACK (field_alias_modified_cb), q);
-	
+
+	query_field_nullify (field);
 	q->fields = g_slist_remove (q->fields, field);
 
 #ifdef debug_signal
@@ -1875,11 +1837,11 @@ query_swap_fields (Query * q, QueryField *f1, QueryField *f2)
 		list2->data = data;
 
 #ifdef debug_signal
-		g_print (">> 'CHANGED' from query_swap_fields\n");
+		g_print (">> 'FIELDS_ORDER_CHANGED' from query_swap_fields\n");
 #endif
-		g_signal_emit (G_OBJECT (q), query_signals[CHANGED], 0);
+		g_signal_emit (G_OBJECT (q), query_signals[FIELDS_ORDER_CHANGED], 0);
 #ifdef debug_signal
-		g_print ("<< 'CHANGED' from query_swap_fields\n");
+		g_print ("<< 'FIELDS_ORDER_CHANGED' from query_swap_fields\n");
 #endif
 	}
 }
@@ -2113,12 +2075,14 @@ query_list_joins (Query *q)
 	GSList *list, *sub_list;
 
 	list = q->joins;
+	g_print ("Top of joins list: %p (%d sub lists)\n",
+		 q->joins, g_slist_length (q->joins));
 	while (list) {
-		g_print ("Master list %p (->%p)\n", list, list->data);
+		g_print ("   Sub list %p (->%p)\n", list, list->data);
 
 		sub_list = (GSList *)(list->data);
 		while (sub_list) {
-			g_print ("\tSub list %p (->%p)\n", sub_list, sub_list->data);
+			g_print ("\tJoin node list %p (->%p)\n", sub_list, sub_list->data);
 			sub_list = g_slist_next (sub_list);
 		}
 		list = g_slist_next (list);
@@ -2160,10 +2124,12 @@ query_print_joins (Query *q)
  * It assumes the list is properly organised (see query_add_join)
  * plists is a pointer to the main list pointer and there should be a sub-list
  * pjoins can be NULL or point to an element of the first sublist
- * *plists will return the main lis element refering to the sub-list
- * cond the view was found (if so) and *pjoins will point to the
- * element in the sub-list cond the join was found.
- * 
+ *
+ * *plists will return the main list element refering to the sub-list
+ * where the view was found (if so) and *pjoins will point to the
+ * element in the sub-list where the join was found.
+ *
+ * The first view which is found causes the function to return. 
  */
 
 static guint
@@ -2171,41 +2137,51 @@ find_views_in_joins_lists (QueryView *vant, QueryView *vsuc, GSList **plists, GS
 {
 	GSList *lists = *plists;
 	GSList *joins = *pjoins;
-	QueryJoin *jelem;
-	QueryView *el_ant, *el_suc;
 	guint found;
 
 	if (*plists == NULL) return FOUND_NONE;
 
 	found = FOUND_NONE;
-	do {
-		if (joins == NULL) { /* search a new sub-list */
-	    		joins = (GSList *) (lists->data);
-			jelem = QUERY_JOIN (joins->data);
-			el_ant = query_join_get_ant_view (jelem);
-			if (vant == el_ant)
+	
+	if (!joins) 
+		joins = (GSList *) (lists->data);
+
+	while (lists && !found) {
+		if (joins) {  /* see if vant or vsuc is found in this join */
+			QueryView *qv;
+			QueryJoin *join;
+
+			join = QUERY_JOIN (joins->data);
+			qv = query_join_get_ant_view (join);
+			if (vant == qv)
 				found = FOUND_VANT;
-			else if (vsuc == el_ant)
-				found = FOUND_VSUC;
-		}
-		else {
-			jelem = QUERY_JOIN (joins->data);
-			el_suc = query_join_get_suc_view (jelem);
-			if (vant == el_suc)
-				found = FOUND_VANT;
-			else if (vsuc == el_suc)
-				found = FOUND_VSUC;
-			if (!found) { /* progress to next element and/or sub-list*/
-				joins = g_slist_next (joins);
-				if (joins == NULL) {
-					lists = g_slist_next (lists);
+			else {
+				if (vsuc == qv)
+					found = FOUND_VSUC;	
+			}
+			if (!found) {
+				qv = query_join_get_suc_view (join);
+				if (vant == qv)
+					found = FOUND_VANT;
+				else {
+					if (vsuc == qv)
+						found = FOUND_VSUC;	
 				}
 			}
 		}
-	} while (lists && !found) ;
+		else {       /* switch to a new joins list */
+			lists = g_slist_next (lists);
+			if (lists)
+				joins = (GSList *) (lists->data);
+		}
+
+		if (!found)  /* switch to the next join in the list */
+			joins = g_slist_next (joins);
+	}
+
+	*plists = lists;	/* sub-list where view was found */
+	*pjoins = joins;	/* join where view was found */
 
-	*plists = lists;	/* sub-list cond view was found */
-	*pjoins = joins;	/* join cond view was found */
 	return found;
 }
 
@@ -2337,7 +2313,7 @@ query_place_list (Query *query, GSList *list, QueryView *view)
  * query, there is no such concept. A visual indication needs to be used
  * in order to signal that the non-joined records of a table should appear.
  * For instance, an asterisk * (a la Oracle) or a small circle, could appear
- * near the table's end of the join line.
+ * near the table's end of the join line (at the moment a crow's foot does appear)
  * 
  * Duplicate joins are not added.
  * 
@@ -2349,8 +2325,8 @@ query_place_list (Query *query, GSList *list, QueryView *view)
  *
  * Disconnected join list are supported. This allows two or more separate
  * groups of tables (now links between groups). This is a cartesian product
- * between the groups). Each group is represented by a G_SList. The query
- * contains a G_SList of these sub-lists.
+ * between the groups). Each group is represented by a GSList. The query
+ * contains a GSList of these sub-lists.
  * 
  * Join list organization: (a join is (ant_view, suc_view, condition) )
  * - 1st join is (t1, t2, c2) and represents "t1 JOIN t2 ON c2"
@@ -2368,7 +2344,7 @@ query_place_list (Query *query, GSList *list, QueryView *view)
  * These sub-lists are separated by commas in the corresponding SQL statement.
  * If a new join is added with one view in a 1st group and the other in a
  * 2nd group, then the two sub-lists have to be properly merged (the order
- * of the joins in the 2nd list usually change and sometimes two views in
+ * of the joins in the 2nd list usually changes and sometimes two views in
  * the same join have to be swapped (in which case a LEFT JOIN becomes a
  * RIGHT JOIN, for instance).
  *
@@ -2388,10 +2364,11 @@ query_place_join (Query *q, QueryJoin *join)
 	g_return_if_fail (vant != vsuc);
 
 	if (q->joins == NULL) {
+		/* no join yet */
 		q->joins = g_slist_prepend (NULL, g_slist_prepend (NULL, join));
 	}
 	else {
-		/* see if one of the join's views are in the joins lists */
+		/* see if one of the join's views is in the joins lists */
 		GSList *list1, *list2;
 		GSList *joins=NULL, *joins2;
 		guint found;
@@ -2403,39 +2380,36 @@ query_place_join (Query *q, QueryJoin *join)
 			q->joins = g_slist_append (q->joins, g_slist_prepend (NULL, join));
 		}
 		else {
-			/* swap views if needed */
 			GSList *elem;
+
+			/* swap views if this is the suc_view we found first */
 			if (found == FOUND_VSUC) {
 				query_join_swap_views (join);
-				vsuc = vant; /* No need for old vsuc! */
+				vsuc = vant; /* No need for old vsuc or for vant! */
 			}
 
-			/* search for the 2nd view, vsuc */
-			joins2 = joins->next;
-			if (joins2 == NULL) {
-				list2 = g_slist_next(list1);
-			}
-			else
-				list2 = list1;
-			if (list2 == NULL)
-				found = FOUND_NONE;
-			else {
-				joins2 = (GSList *) list2->data;
-				found = find_views_in_joins_lists (NULL, vsuc, &list2, &joins2);
-			}
+			/* search for the 2nd view, vsuc:
+			   set joins2 and list2 for starting point of next search */
+			joins2 = g_slist_next (joins);
+			list2 = list1;
+			if (!joins2)
+				list2 = g_slist_next (list1);
+			
+			found = find_views_in_joins_lists (NULL, vsuc, &list2, &joins2);
 
-			if ((list1 == list2) && joins2) { /* found loop in the list */
-				query_join_set_skip_view(join);
-				joins = joins2;
-			}
+			if (!found)
+				if ((list1 == list2) && joins2) { /* found loop in the list */
+					query_join_set_skip_view (join);
+					joins = joins2;
+				}
 
-			/* - insert new join after joins position */
+			/* insert new join after joins position */
 			elem = g_slist_prepend (NULL, join);
 			elem->next = joins->next;
 			joins->next = elem;
 
 			if (found != FOUND_NONE) {
-				if (list1 != list2) { /* have merge two sub-lists */
+				if (list1 != list2) { /* we have to merge two sub-lists */
 					q->joins = g_slist_remove_link (q->joins, list2);
 					query_place_list (q, list2->data, vsuc);
 					g_slist_free_1 (list2);
@@ -2570,9 +2544,10 @@ real_query_add_join (Query * q, QueryJoin *join)
 static void 
 join_nullified_cb (QueryJoin *join, Query *q)
 {
-	GSList *lists, *antl;
-	GSList *joins, *antj;
+	GSList *toplist;
+	GSList *joins;
 	GSList *slist2, *tmp;
+	gint pos;
 
 	g_return_if_fail (q && IS_QUERY (q));
 	g_return_if_fail (join && IS_QUERY_JOIN (join));
@@ -2591,66 +2566,55 @@ join_nullified_cb (QueryJoin *join, Query *q)
 
 
 	/* search for the join in q->joins */
-	lists = q->joins;	/* points to a sub-list */
-	antl = lists;		/* will point to the sub-list before lists */
-	joins = (GSList *) (lists->data);
-	antj = joins;		/* will point to the element before joins */
+	toplist = q->joins;	
+	joins = (GSList *) (toplist->data);
 
-	while (lists && (join != joins->data)) {
-		antj = joins;
+	while (toplist && joins && (joins->data != join)) {
 		joins = g_slist_next (joins);
 		if (! joins) { /* search in a new sub-list */
-			antl = lists;
-			lists = g_slist_next (lists);
-			if (lists) {
-				joins = (GSList *) (lists->data);
-				antj = joins;
-			}
+			toplist = g_slist_next (toplist);
+			if (toplist)
+				joins = (GSList *) (toplist->data);
 		}
 	}
 	
 	/* if the join has not been found, return with a warning */
-        g_assert (lists);
+        g_return_if_fail (toplist && joins);
 
-	/* remove the join from the list */
-	slist2 = joins->next;	/* holds join sub-list after the join item */
-	if (antj == joins) { /* it's the first join in the list */ 
-		if (! g_slist_next (joins)) { /* there is no other join */
-			/* the sub-list becomes empty and has to be removed */
-			if (antl == lists) 	/* it's the first sub-list */
-				q->joins = lists->next;
-			else
-				antl->next = lists->next;
-			g_slist_free_1 (lists);
-		}
+	/* from now:
+	   joins->data == join
+	   "toplist" is a list (a node of q->joins) containing the "joins" node
+	*/
+
+	/* the idea is to split toplist->data in two parts:
+	   -> the one before "joins": for this a new list is created to copy that
+	      part and re inserted where "toplist" was
+	   -> the one after "joins": every QueryJoin in this part is reinserted
+  	      using query_place_join() to correctly manage the lists of lists of joins
+	*/
+	pos = g_slist_index (q->joins, toplist);
+	q->joins = g_slist_remove_link (q->joins, toplist);
+	/* part before "joins" */
+	slist2 = NULL;
+	for (tmp = toplist->data; tmp != joins; tmp = g_slist_next (tmp)) 
+		slist2 = g_slist_append (slist2, tmp->data);
+	if (slist2)
+		q->joins = g_slist_insert (q->joins, slist2, pos);
+
+	/* part after "joins" */
+	tmp = g_slist_next (tmp);
+	while (tmp) {
+		query_join_unset_skip_view (QUERY_JOIN (tmp->data));
+		query_place_join (q, QUERY_JOIN (tmp->data));
+		tmp = g_slist_next (tmp);
 	}
-
-	if (query_join_skip_view (QUERY_JOIN(joins->data))) {
-		/* the join _identifies_ a loop; just remove it */
-		antj->next = joins->next;
-	}
-	else {
-		if (slist2 && query_join_skip_view (QUERY_JOIN(slist2->data))) {
-			/* the join is _part_ of a loop; just remove it */
-			antj->next = joins->next;
-			/* the next join identifies a loop; unset it */
-			query_join_unset_skip_view (QUERY_JOIN(slist2->data));
-		}
-		else {
-			/* split the list and re-place the 2nd half in the query */
-			antj->next = NULL;
-			while (slist2) {
-				query_join_unset_skip_view (QUERY_JOIN (slist2->data));
-				query_place_join (q, QUERY_JOIN (slist2->data));
-				tmp = slist2;
-				slist2 = g_slist_next (slist2);
-				g_slist_free_1 (tmp);
-			}
-		}
-	}
-
-	g_slist_free_1 (joins);
-
+	g_slist_free (toplist->data);
+	g_slist_free (toplist);
+	
+#ifdef debug
+	g_print ("join_nullified_cb (%p): new organisation:\n", join);
+	query_list_joins (q);
+#endif
 
 #ifdef debug_signal
 	g_print (">> 'JOIN_DROPPED' from query_del_join\n");
@@ -2698,12 +2662,15 @@ query_del_join (Query * q, QueryJoin *join)
  * Query COND/HAVING management
  */
 
-/* WARNING: cond MUST not already be the child of another QueryCond */
+static void cond_nullified_cb (QueryCond *cond, Query *q);
+static void cond_changed_cb (QueryCond *cond, Query *q);
+
 void 
 query_add_cond (Query * q, QueryCond *cond, QueryCond * parent, QueryCond * sibling)
 {
 	gint pos = -1;
 
+	/* arguments validity tests */
 	g_return_if_fail (q && IS_QUERY (q));
 	g_return_if_fail (cond && IS_QUERY_COND (cond));
 	g_return_if_fail (query_cond_get_query (cond) == q);
@@ -2711,7 +2678,9 @@ query_add_cond (Query * q, QueryCond *cond, QueryCond * parent, QueryCond * sibl
 	if (parent) {
 		g_return_if_fail (IS_QUERY_COND (parent));
 		g_return_if_fail (query_cond_get_query (parent) == q);
-		g_return_if_fail (q->where_cond && query_cond_is_ancestor (parent, q->where_cond));
+		g_return_if_fail (q->where_cond && 
+				  query_cond_is_ancestor (parent, q->where_cond));
+		g_return_if_fail (query_cond_get_cond_type (parent) != QUERY_COND_COND);
 	}
 
 	if (sibling) {
@@ -2722,48 +2691,99 @@ query_add_cond (Query * q, QueryCond *cond, QueryCond * parent, QueryCond * sibl
 		pos = g_slist_index (query_cond_get_children (parent), sibling);
 	}
 
-	if (parent) 
+	/* actual insertion */
+	if (parent) {
 		query_cond_add_to_node (parent, cond, pos);
-	else {
-		if (q->where_cond)
-			query_cond_add_to_node (q->where_cond, cond, -1);
-		else
-			q->where_cond = cond;
 	}
+	else {
+		if (q->where_cond) {
+			if (query_cond_get_cond_type (q->where_cond) != QUERY_COND_COND)
+				query_cond_add_to_node (q->where_cond, cond, -1);
+			else {
+				QueryCond *oldhead = q->where_cond;
+				QueryCond *and = QUERY_COND (query_cond_new_as_node (q, QUERY_COND_AND));
+				g_signal_handlers_disconnect_by_func (G_OBJECT (oldhead),
+								      G_CALLBACK (cond_nullified_cb), q);
+				g_signal_handlers_disconnect_by_func (G_OBJECT (oldhead),
+								      G_CALLBACK (cond_changed_cb), q);
+
+				q->where_cond = and;
+				g_object_ref (G_OBJECT (and));
+
+				query_cond_add_to_node (and, oldhead, -1);
+				g_object_unref (G_OBJECT (oldhead));
+
+				g_signal_connect (G_OBJECT (and), "nullified",
+						  G_CALLBACK (cond_nullified_cb), q);
+				g_signal_connect (G_OBJECT (and), "changed",
+						  G_CALLBACK (cond_changed_cb), q);
+				query_cond_add_to_node (and, cond, -1);
+			}
+		}
+		else {
+			q->where_cond = cond;
+			g_object_ref (G_OBJECT (cond));
+
+			g_signal_connect (G_OBJECT (cond), "nullified",
+					  G_CALLBACK (cond_nullified_cb), q);
+			g_signal_connect (G_OBJECT (cond), "changed",
+					  G_CALLBACK (cond_changed_cb), q);
 
-	
 #ifdef debug_signal
-	g_print (">> 'COND_CREATED' from query_add_cond\n");
+			g_print (">> 'COND_CHANGED' from query_add_cond\n");
 #endif
-	g_signal_emit (G_OBJECT (q), query_signals[COND_CREATED], 0, cond);
+			g_signal_emit (G_OBJECT (q), query_signals[COND_CHANGED], 0);
 #ifdef debug_signal
-	g_print ("<< 'COND_CREATED' from query_add_cond\n");
+			g_print ("<< 'COND_CHANGED' from query_add_cond\n");
 #endif	
-	/* FIXME: catch signals from the QueryCond object */
+		}
+	}
 }
 
+static void
+cond_nullified_cb (QueryCond *cond, Query *q)
+{
+	query_del_cond (q, cond);
+}
+
+static void
+cond_changed_cb (QueryCond *cond, Query *q)
+{
+#ifdef debug_signal
+	g_print (">> 'COND_CHANGED' from cond_changed_cb\n");
+#endif
+	g_signal_emit (G_OBJECT (q), query_signals[COND_CHANGED], 0);
+#ifdef debug_signal
+	g_print (">> 'COND_CHANGED' from cond_changed_cb\n");
+#endif	
+}
+
+
 void 
 query_del_cond (Query * q, QueryCond *cond)
 {
-	QueryCond *parent;
+	QueryCond *parent = NULL;
 
 	g_return_if_fail (q && IS_QUERY (cond));
 	g_return_if_fail (cond && IS_QUERY_COND (cond));
 	g_return_if_fail (query_cond_get_query (cond) == q);
 	g_return_if_fail (q->where_cond);
-
-	parent = query_cond_get_parent (cond);
-	g_return_if_fail (parent);
-
-	query_cond_del_from_node (parent, cond);
+	
+	if (cond != q->where_cond) {
+		parent = query_cond_get_parent (cond);
+		g_assert (parent);
+		query_cond_del_from_node (parent, cond);
+	}
+	else {
+		g_signal_handlers_disconnect_by_func (G_OBJECT (q->where_cond),
+						      G_CALLBACK (cond_nullified_cb), q);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (q->where_cond),
+						      G_CALLBACK (cond_changed_cb), q);
 		
-#ifdef debug_signal
-	g_print (">> 'COND_DROPPED' from query_del_cond\n");
-#endif
-	g_signal_emit (G_OBJECT (q), query_signals[COND_DROPPED], 0, cond);
-#ifdef debug_signal
-	g_print ("<< 'COND_DROPPED' from query_del_cond\n");
-#endif
+		q->where_cond = NULL;
+		g_object_unref (G_OBJECT (q->where_cond));
+		cond_changed_cb (NULL, q);
+	}
 }
 
 void 
@@ -2777,14 +2797,6 @@ query_move_cond (Query * q, QueryCond * cond, QueryCond * to_parent, QueryCond *
 	query_del_cond (q, cond);
 	query_add_cond (q, cond, to_parent, to_sibling);
 	g_object_unref (G_OBJECT (cond));
-
-#ifdef debug_signal
-	g_print (">> 'COND_MOVED' from query_move_cond\n");
-#endif
-	g_signal_emit (G_OBJECT (q), query_signals[COND_MOVED], 0, cond);
-#ifdef debug_signal
-	g_print ("<< 'COND_MOVED' from query_move_cond\n");
-#endif
 }
 
 
@@ -3037,7 +3049,7 @@ query_get_select_query (Query * q, GSList * missing_values)
 		/* COND conditions */
 		str = get_cond_part_of_sql (q, missing_values);
 		if (str) {
-			g_string_sprintfa (SQL, "\nCOND\n%s", str);
+			g_string_sprintfa (SQL, "\nWHERE\n%s", str);
 			g_free (str);
 		}
 
@@ -3098,35 +3110,13 @@ static gchar *
 get_cond_part_of_sql (Query * q, GSList * missing_values)
 {
 	gboolean firstnode = TRUE;
-	GString *gstr;
-	gchar *wh = NULL;
+	gchar *str = NULL;
 
-	gstr = g_string_new ("");
-	/* FIXME: VIV if (q->where_tree->children) { */
-/* 		GNode *gnode = q->where_tree->children; */
-
-/* 		while (gnode) { */
-/* 			if (firstnode) */
-/* 				firstnode = FALSE; */
-/* 			else */
-/* 				g_string_append (gstr, "AND "); */
-/* 			wh = query_cond_render_as_sql (QUERY_COND (gnode->data), */
-/* 							    missing_values); */
-/* 			g_string_append (gstr, wh); */
-/* 			g_free (wh); */
-/* 			g_string_append (gstr, "\n"); */
-/* 			gnode = gnode->next; */
-/* 		} */
-/* 	} */
-
-	if (!firstnode) {
-		wh = gstr->str;
-		g_string_free (gstr, FALSE);
+	if (q->where_cond) {
+		str = query_cond_render_as_sql (q->where_cond, missing_values);
 	}
-	else 
-		g_string_free (gstr, TRUE);
 
-	return wh;
+	return str;
 }
 
 
@@ -3193,7 +3183,7 @@ get_from_part_of_sql (Query * q)
 			else
 				str_cond = g_strdup ("???");
 			
-			if (first) {	/* Ex: "tableAnt INNER JOIN tableSuc ON condition" */
+			if (first) { /* Ex: "tableAnt INNER JOIN tableSuc ON condition" */
 				first = FALSE;
 
 				g_string_sprintf (joins, "%s %s %s ON %s",
@@ -3202,16 +3192,18 @@ get_from_part_of_sql (Query * q)
 						  query_view_get_alias_expr (el_suc),
 						  str_cond);
 			}
-			else if (query_join_skip_view (jelem))
-				/* Ex: "... AND condition" */
-				g_string_sprintfa (joins, " AND %s",
-						   str_cond);
-			else
-				/* Ex: "... INNER_JOIN tableSuc ON condition" */
-				g_string_sprintfa (joins, " %s %s ON %s",
-						   query_join_get_join_type_sql (jelem),
-						   query_view_get_alias_expr (el_suc),
-						   str_cond);
+			else {
+				if (query_join_skip_view (jelem))
+					/* Ex: "... AND condition" */
+					g_string_sprintfa (joins, " AND %s",
+							   str_cond);
+				else
+					/* Ex: "... INNER_JOIN tableSuc ON condition" */
+					g_string_sprintfa (joins, " %s %s ON %s",
+							   query_join_get_join_type_sql (jelem),
+							   query_view_get_alias_expr (el_suc),
+							   str_cond);
+			}
 
 			g_free (str_cond);
 
@@ -3400,7 +3392,17 @@ static_query_dump_contents (Query *q, guint offset)
 			}
 			else
 				g_print ("NONE\n");
-
+			if (query_field_get_ftype (qf) == QUERY_FIELD_VALUE) {
+				const GdaValue *val;
+				g_print ("%s\tVALUE: ServerDataType GDA value = %d\n", str,
+					 query_field_value_get_data_type (qf)->gda_type);
+				val = query_field_value_get_value (qf);
+				if (val)
+					g_print ("%s\tVALUE: GdaValue = %p, type = %d\n", str, val,
+						 gda_value_get_type (val));
+				else
+					g_print ("%s\tVALUE: GdaValue = %p\n", str, val);
+			}
 			if (query_field_get_activated (qf)) {
 				gchar *sql;
 
@@ -3422,7 +3424,7 @@ static_query_dump_contents (Query *q, guint offset)
 	if (!list)
 		g_print ("%s\tNONE\n", str);
 	while (list) {
-		g_print ("%s" D_COL_H2 "\tMaster list " D_COL_NOR "%p (->%p)\n", str, list, list->data);
+		g_print ("%s\t" D_COL_H2 "Master Node " D_COL_NOR "%p (%d joins)\n", str, list, g_slist_length (list->data));
 		sub_list = (GSList*)(list->data);
 		while (sub_list) {
 			gchar *str_cond;
@@ -3432,12 +3434,13 @@ static_query_dump_contents (Query *q, guint offset)
 				str_cond = query_cond_render_as_sql (query_join_get_condition (qj), NULL);
 			else
 				str_cond = g_strdup ("???");
-			g_print ("%s\t%p->Join: %s\n", str, sub_list, str_cond);
+			g_print ("%s\t%p->Join: %p = %s\n", str, sub_list, sub_list->data,
+				 str_cond);
 			g_free (str_cond);
 
 			/* Join condition details */
 			if (query_join_get_condition (qj))
-				query_cond_dump_contents (query_join_get_condition (qj), offset+5);
+				query_cond_dump_contents (query_join_get_condition (qj), offset+8);
 
 			sub_list = g_slist_next (sub_list);
 		}
