@@ -1,6 +1,6 @@
 /* relship-view.c
  *
- * Copyright (C) 2002 Vivien Malerba
+ * Copyright (C) 2002 - 2003 Vivien Malerba
  * Copyright (C) 2002 Fernando Martins
  *
  * This program is free software; you can redistribute it and/or
@@ -97,13 +97,11 @@ relship_view_init (RelShipView * rs)
 {
 	rs->query = NULL;
 	rs->items = NULL;
-	rs->background = NULL;
 
 	rs->xmouse = 50.;
 	rs->ymouse = 50.;
 }
 
-static void widget_size_changed_cb(GtkWidget *wid, GtkAllocation *alloc, RelShipView *rs);
 static int canvas_event(GnomeCanvasItem *item, GdkEvent *event, RelShipView * rs);
 GtkWidget *
 relship_view_new (Query *q) 
@@ -121,45 +119,20 @@ relship_view_new (Query *q)
 	rsv->query = q;
 	relship_view_post_init (rsv);
 
-	g_signal_connect (G_OBJECT (rsv),"size_allocate",
-			  G_CALLBACK(widget_size_changed_cb), rsv);
-
-	g_signal_connect (G_OBJECT (rsv->background),"event",
+	g_signal_connect (G_OBJECT (rsv),"event-after",
 			  G_CALLBACK(canvas_event), rsv);
 
 	return GTK_WIDGET (obj);
 }
 
-/* sets the background canvas item to occupy at least the whole window so the
-   user can't click outside of it. */ 
-static void widget_size_changed_cb(GtkWidget *wid, GtkAllocation *alloc, RelShipView *rsv)
-{
-	double wx1, wy1, wx2, wy2;
-	double sx1, sy1, sx2, sy2;
-
-	gnome_canvas_get_scroll_region (GNOME_CANVAS (rsv), &sx1, &sy1, &sx2, &sy2);
-	gnome_canvas_window_to_world (GNOME_CANVAS (rsv), 0., 0., &wx1, &wy1);
-	gnome_canvas_window_to_world (GNOME_CANVAS (rsv), alloc->width, alloc->height, &wx2, &wy2);
-
-	if (wx1 > sx1) wx1 = sx1;
-	if (wy1 > sy1) wy1 = sy1;
-	if (wx2 < sx2) wx2 = sx2;
-	if (wy2 < sy2) wy2 = sy2;
-	
-	gnome_canvas_item_set (rsv->background, 
-			       "x1", wx1,
-			       "y1", wy1,
-			       "x2", wx2,
-			       "y2", wy2,
-			       NULL);
-}
-
-
 static GtkWidget *build_context_menu (RelShipView * rsv);
 static int 
 canvas_event(GnomeCanvasItem *item, GdkEvent *event, RelShipView * rsv)
 {
 	gboolean done = TRUE;
+	GnomeCanvasItem *itemat;
+	gdouble wx, wy;
+
 
 	if (g_object_get_data (G_OBJECT (gnome_canvas_root (GNOME_CANVAS (rsv))), "dragged_from")) {
 		/* Dragging cancelled */
@@ -168,24 +141,31 @@ canvas_event(GnomeCanvasItem *item, GdkEvent *event, RelShipView * rsv)
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
-		if (((GdkEventButton *) event)->button == 3) {
-			GtkWidget *menu;
+		gnome_canvas_window_to_world (GNOME_CANVAS (rsv), 
+					      ((GdkEventButton *) event)->x, 
+					      ((GdkEventButton *) event)->y,
+					      &wx, &wy);
+		itemat = gnome_canvas_get_item_at (GNOME_CANVAS (rsv), wx, wy);
 
-			rsv->xmouse = ((GdkEventButton *) event)->x;
-			rsv->ymouse = ((GdkEventButton *) event)->y;
-			menu = build_context_menu (rsv);
-			gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-					NULL, NULL, ((GdkEventButton *)event)->button,
-					((GdkEventButton *)event)->time);
-			
-			done = FALSE;
-			break;
+		if (!itemat) {
+			if (((GdkEventButton *) event)->button == 3) {
+				GtkWidget *menu;
+				
+				rsv->xmouse = wx;
+				rsv->ymouse = wy;
+				menu = build_context_menu (rsv);
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+						NULL, NULL, ((GdkEventButton *)event)->button,
+						((GdkEventButton *)event)->time);
+				
+				done = TRUE;
+			}
 		}
+		break;
 	default:
 		done = FALSE;
 		break;
 	}
-
 	return done;	
 }
 
@@ -337,17 +317,6 @@ relship_view_post_init  (RelShipView * rsv)
 	gdouble x1, y1, x2, y2;
 
 	gnome_canvas_get_scroll_region (GNOME_CANVAS (rsv), &x1, &y1, &x2, &y2);
-	rsv->background = gnome_canvas_item_new (gnome_canvas_root (GNOME_CANVAS (rsv)),
-						GNOME_TYPE_CANVAS_RECT,
-						"x1", 0.,
-						"y1", 0.,
-						"x2", 10.,
-						"y2", 10.,
-						"fill_color", "gray90",
-						"outline_color", "black",
-						"width_units", 1.0,
-						NULL);
-	gnome_canvas_item_lower_to_bottom (rsv->background);
 
 	/* display all the QueryViews which are present */
 	list = rsv->query->views;
@@ -427,8 +396,8 @@ item_moved_cb (CanvasBase *item, RelShipView *rs)
 	double sx1, sy1, sx2, sy2;
 	gboolean changed = FALSE;
 
-	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (item), &ix1, &iy1, &ix2, &iy2);
-	gnome_canvas_get_scroll_region (GNOME_CANVAS (rs), &sx1, &sy1, &sx2, &sy2);
+	gnome_canvas_get_scroll_region (GNOME_CANVAS (rs), &sx1, &sy1, &sx2, &sy2); /* World coord. */
+	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (item), &ix1, &iy1, &ix2, &iy2); /* rs's parent coord. */
 
 	if (ix1 < sx1) {
 		sx1 = ix1 - 2.;
@@ -451,16 +420,7 @@ item_moved_cb (CanvasBase *item, RelShipView *rs)
 	}
 
 	if (changed) {
-		/* FIXME: there is a bug in the canvas because this function displays trash */
 		gnome_canvas_set_scroll_region (GNOME_CANVAS (rs), sx1, sy1, sx2, sy2);
-		
-		/* keep the background layer at a good dimension */
-		gnome_canvas_item_set (rs->background, 
-				       "x1", sx1,
-				       "y1", sy1,
-				       "x2", sx2,
-				       "y2", sy2,
-				       NULL);
 	}
 
 #ifdef debug_signal
